# (C) 2004 Dieter Menne, http://www.menne-biomed.de/gastempt
# Auxillary functions for the gastric emptying analysis examples

# Self starting functions used to fit gastric emptying curves
EmptInit= function(mCall,LHS,data)
{
  model=as.character(mCall[[1]])
  xy=sortedXyData(mCall[["t"]],LHS,data)
  if (nrow(xy) < 6) {
    .Exit("Too few distinct input values to fit tmodel")
  }
  # we use the first of the sorted data points for
  # the initial volume (v0) estimation
  v0 = xy[1,2]
  xy$y = log(xy$y/v0)
  tempt = log(0.5)/coef(lm(y~x,data=xy))[2]
  if (length(grep("Lin",model)))
     tempt=tempt/2 #empirical correction for LinExp
  tempt = max(min(tempt,200),20)
  kappa = 0.8 # a good starting value for LinCxp
  beta  = 1 # try with simple exponential first
  if (length(grep("Log",model))) {
    tempt = log(tempt)
    kappa = log(kappa)
  }
  if (length(grep("Lin",model))) {
     value = c(v0,kappa,tempt)
   } else {
     value = c(v0,beta,tempt)
  }
  ca = switch(model,
    SSEmptLinExp     = c("kappa","tempt"),
    SSEmptLinExpLog  = c("logkappa","logtempt"),
    SSEmptPowExp     = c("beta","tempt"),
    SSEmptPowExpLog  = c("logbeta","logtempt"))
  names(value) = mCall[c("v0",ca)]
  value
}

# Straight Parameters
SSEmptLinExp=selfStart(~v0*(1+kappa*t/tempt)*exp(-t/tempt),
  initial=EmptInit, parameters= c("v0","kappa","tempt"))

# Power Exponential self starting model
SSEmptPowExp=selfStart(~v0*exp(-(t/tempt)^beta),
  initial=EmptInit, parameters= c("v0","beta","tempt"))

# Log-transformed parameters
# Should be used if to force both kappa and empt
SSEmptLinExpLog=selfStart(~v0*(1+exp(logkappa)*t/exp(logtempt))*exp(-t/exp(logtempt)),
  initial=EmptInit, parameters= c("v0","logkappa","logtempt"))

SSEmptPowExpLog=selfStart(~v0*exp(-(t/exp(logtempt))^exp(logbeta)),
  initial=EmptInit, parameters= c("v0","logbeta","logtempt"))

# Determine t(50%) from parameters
# No closed solution known for LinExp, we use newton  approximation
# Don't forget to change the low/up values if you should
# measure gastric emptying in hours or microseconds
t50 = function(x){
   # we expect times between 10 and 2000 minutes
  interval=c(20,2000)
  if (names(x)[2] == "logbeta"){
    value= uniroot(function(t)
       SSEmptPowExpLog(t,1,x["logbeta"],x["logtempt"]) - 0.5,
       interval=interval)
  } else
  if (names(x)[2] == "beta"){
    value= uniroot(function(t)
       SSEmptPowExp(t,1,x["beta"],x["tempt"]) - 0.5,
       interval=interval)
  } else
  if (names(x)[2] == "logkappa"){
    value= uniroot(function(t)
       SSEmptLinExpLog(t,1,x["logkappa"],x["logtempt"]) - 0.5,
       interval=interval)
  } else
  if (names(x)[2] == "kappa"){
  value= uniroot(function(t)
     SSEmptLinExp(t,1,x["kappa"],x["tempt"]) - 0.5,
     interval=interval)
  }
  value$root
}
